@@ -204,7 +204,7 @@ main(int argc, char *argv[])
 	    if (clp->negated)
 		errh = ErrorHandler::default_handler();
 	    else
-		errh = ErrorHandler::silent_handler();
+		errh = new SilentErrorHandler;
 	    break;
       
 	  case VERSION_OPT:
