@@ -1,6 +1,6 @@
 /* otfinfo.cc -- driver for reporting information about OpenType fonts
  *
- * Copyright (c) 2003-2004 Eddie Kohler
+ * Copyright (c) 2003-2005 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -49,20 +49,34 @@ using namespace Efont;
 #define QUERY_FEATURES_OPT	321
 #define QUERY_OPTICAL_SIZE_OPT	322
 #define QUERY_POSTSCRIPT_NAME_OPT 323
+#define QUERY_GLYPHS_OPT	324
+#define QUERY_FVERSION_OPT	325
+#define TABLES_OPT		326
+#define QUERY_FAMILY_OPT	327
 
 Clp_Option options[] = {
     
     { "script", 0, SCRIPT_OPT, Clp_ArgString, 0 },
     { "quiet", 'q', QUIET_OPT, 0, Clp_Negate },
     { "verbose", 'V', VERBOSE_OPT, 0, Clp_Negate },
-    { "query-features", 'f', QUERY_FEATURES_OPT, 0, 0 },
+    { "features", 'f', QUERY_FEATURES_OPT, 0, 0 },
+    { "scripts", 's', QUERY_SCRIPTS_OPT, 0, 0 },
+    { "optical-size", 'z', QUERY_OPTICAL_SIZE_OPT, 0, 0 },
+    { "postscript-name", 'p', QUERY_POSTSCRIPT_NAME_OPT, 0, 0 },
+    { "family", 'a', QUERY_FAMILY_OPT, 0, 0 },
+    { "font-version", 'v', QUERY_FVERSION_OPT, 0, 0 },
+    { "glyphs", 'g', QUERY_GLYPHS_OPT, 0, 0 },
+    { "tables", 't', TABLES_OPT, 0, 0 },
+    { "help", 'h', HELP_OPT, 0, 0 },
+    { "version", 0, VERSION_OPT, 0, 0 },
+    // old synonyms
+    { "query-features", 0, QUERY_FEATURES_OPT, 0, 0 },
     { "qf", 0, QUERY_FEATURES_OPT, 0, 0 },
     { "query-scripts", 's', QUERY_SCRIPTS_OPT, 0, 0 },
     { "qs", 0, QUERY_SCRIPTS_OPT, 0, 0 },
     { "query-optical-size", 'z', QUERY_OPTICAL_SIZE_OPT, 0, 0 },
     { "query-postscript-name", 'p', QUERY_POSTSCRIPT_NAME_OPT, 0, 0 },
-    { "help", 'h', HELP_OPT, 0, 0 },
-    { "version", 0, VERSION_OPT, 0, 0 },
+    { "query-glyphs", 'g', QUERY_GLYPHS_OPT, 0, 0 }
     
 };
 
@@ -94,25 +108,29 @@ usage()
 {
     printf("\
 'Otfinfo' reports information about an OpenType font to standard output.\n\
-Supply a '--query' to say what information you want.\n\
+Options specify what information to print.\n\
 \n\
-Usage: %s QUERY_OPTION [OTFFILES...]\n\n",
+Usage: %s [-sfzpg] [OTFFILES...]\n\n",
 	   program_name);
     printf("\
 Query options:\n\
-  -s, --query-scripts          Report font's supported scripts.\n\
-  -f, --query-features         Report font's GSUB/GPOS features.\n\
-  -z, --query-optical-size     Report font's optical size information.\n\
-  -p, --query-postscript-name  Report font's PostScript name.\n\
+  -s, --scripts                Report font's supported scripts.\n\
+  -f, --features               Report font's GSUB/GPOS features.\n\
+  -z, --optical-size           Report font's optical size information.\n\
+  -p, --postscript-name        Report font's PostScript name.\n\
+  -a, --family                 Report font's family name.\n\
+  -v, --font-version           Report font's version information.\n\
+  -g, --glyphs                 Report font's glyph names.\n\
+  -t, --tables                 Report font's OpenType tables.\n\
 \n\
 Other options:\n\
-      --script=SCRIPT[.LANG]   Set script used for --query-features [latn].\n\
+      --script=SCRIPT[.LANG]   Set script used for --features [latn].\n\
   -V, --verbose                Print progress information to standard error.\n\
   -h, --help                   Print this message and exit.\n\
   -q, --quiet                  Do not generate any error messages.\n\
       --version                Print version number and exit.\n\
 \n\
-Report bugs to <kohler@icir.org>.\n");
+Report bugs to <kohler@cs.ucla.edu>.\n");
 }
 
 String
@@ -181,7 +199,7 @@ do_query_scripts(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *re
 {
     Vector<String> results;
     if (String gsub_table = otf.table("GSUB")) {
-	OpenType::Gsub gsub(gsub_table, errh);
+	OpenType::Gsub gsub(gsub_table, &otf, errh);
 	collect_script_descriptions(gsub.script_list(), results, errh);
     }
     if (String gpos_table = otf.table("GPOS")) {
@@ -219,7 +237,7 @@ do_query_features(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *r
 {
     Vector<String> results;
     if (String gsub_table = otf.table("GSUB")) {
-	OpenType::Gsub gsub(gsub_table, errh);
+	OpenType::Gsub gsub(gsub_table, &otf, errh);
 	collect_feature_descriptions(gsub.script_list(), gsub.feature_list(), results, errh);
     }
     if (String gpos_table = otf.table("GPOS")) {
@@ -280,6 +298,22 @@ do_query_optical_size(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandle
     }
 }
 
+static void
+do_query_family_name(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh)
+{
+    int before_nerrors = errh->nerrors();
+    String family_name = "no family name information";
+
+    if (String name_table = otf.table("name")) {
+	OpenType::Name name(name_table, errh);
+	if (name.ok())
+	    family_name = name.name(std::find_if(name.begin(), name.end(), OpenType::Name::EnglishPlatformPred(OpenType::Name::N_FAMILY)));
+    }
+
+    if (errh->nerrors() == before_nerrors)
+	result_errh->message("%s", family_name.c_str());
+}
+
 static void
 do_query_postscript_name(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh)
 {
@@ -296,6 +330,70 @@ do_query_postscript_name(const OpenType::Font &otf, ErrorHandler *errh, ErrorHan
 	result_errh->message("%s", postscript_name.c_str());
 }
 
+static void
+do_query_font_version(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh)
+{
+    int before_nerrors = errh->nerrors();
+    String version = "no version information";
+
+    if (String name_table = otf.table("name")) {
+	OpenType::Name name(name_table, errh);
+	if (name.ok())
+	    version = name.name(std::find_if(name.begin(), name.end(), OpenType::Name::EnglishPlatformPred(OpenType::Name::N_VERSION)));
+    }
+
+    if (errh->nerrors() == before_nerrors)
+	result_errh->message("%s", version.c_str());
+}
+
+static void
+do_query_glyphs(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh)
+{
+    int before_nerrors = errh->nerrors();
+    try {
+	// get font
+	Cff cff(otf.table("CFF"), errh);
+	if (!cff.ok())
+	    throw OpenType::Error();
+
+	Cff::FontParent *fp = cff.font(PermString(), errh);
+	if (!fp || !fp->ok())
+	    throw OpenType::Error();
+	Cff::Font *font = dynamic_cast<Cff::Font *>(fp);
+	if (!font) {
+	    errh->error("CID-keyed fonts not supported");
+	    throw OpenType::Error();
+	}
+
+	// save glyph names
+	Vector<PermString> glyph_names;
+	font->glyph_names(glyph_names);
+	for (PermString* s = glyph_names.begin(); s < glyph_names.end(); s++)
+	    result_errh->message("%s", s->c_str());
+    } catch (OpenType::Error) {
+	if (errh->nerrors() == before_nerrors)
+	    result_errh->message("no glyph name information");
+    }
+}
+
+static void
+do_tables(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh)
+{
+    int before_nerrors = errh->nerrors();
+    try {
+	int n = otf.ntables();
+
+	for (int i = 0; i < n; i++)
+	    if (OpenType::Tag tag = otf.table_tag(i)) {
+		String s = otf.table(tag);
+		result_errh->message("%7u %s\n", s.length(), tag.text().c_str());
+	    }
+    } catch (OpenType::Error) {
+	if (errh->nerrors() == before_nerrors)
+	    result_errh->message("corrupted tables");
+    }
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -336,8 +434,12 @@ main(int argc, char *argv[])
 	  case QUERY_FEATURES_OPT:
 	  case QUERY_OPTICAL_SIZE_OPT:
 	  case QUERY_POSTSCRIPT_NAME_OPT:
+	  case QUERY_GLYPHS_OPT:
+	  case QUERY_FAMILY_OPT:
+	  case QUERY_FVERSION_OPT:
+	  case TABLES_OPT:
 	    if (query)
-		usage_error(errh, "supply exactly one --query option");
+		usage_error(errh, "supply exactly one query type option");
 	    query = opt;
 	    break;
 
@@ -345,7 +447,7 @@ main(int argc, char *argv[])
 	    if (clp->negated)
 		errh = ErrorHandler::default_handler();
 	    else
-		errh = ErrorHandler::silent_handler();
+		errh = new SilentErrorHandler;
 	    break;
 
 	  case VERBOSE_OPT:
@@ -354,7 +456,7 @@ main(int argc, char *argv[])
 
 	  case VERSION_OPT:
 	    printf("otfinfo (LCDF typetools) %s\n", VERSION);
-	    printf("Copyright (C) 2003-2004 Eddie Kohler\n\
+	    printf("Copyright (C) 2003-2005 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
@@ -385,7 +487,7 @@ particular purpose.\n");
     
   done:
     if (!query)
-	usage_error(errh, "supply exactly one --query option");
+	usage_error(errh, "supply exactly one query option");
     if (!input_files.size())
 	input_files.push_back("-");
     if (script.null())
@@ -414,6 +516,14 @@ particular purpose.\n");
 	    do_query_optical_size(otf, &cerrh, result_errh);
 	else if (query == QUERY_POSTSCRIPT_NAME_OPT)
 	    do_query_postscript_name(otf, &cerrh, result_errh);
+	else if (query == QUERY_GLYPHS_OPT)
+	    do_query_glyphs(otf, &cerrh, result_errh);
+	else if (query == QUERY_FAMILY_OPT)
+	    do_query_family_name(otf, &cerrh, result_errh);
+	else if (query == QUERY_FVERSION_OPT)
+	    do_query_font_version(otf, &cerrh, result_errh);
+	else if (query == TABLES_OPT)
+	    do_tables(otf, &cerrh, result_errh);
     }
     
     return (errh->nerrors() == 0 ? 0 : 1);
