@@ -1,6 +1,6 @@
 /* dvipsencoding.{cc,hh} -- store a DVIPS encoding
  *
- * Copyright (c) 2003-2004 Eddie Kohler
+ * Copyright (c) 2003-2005 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -18,6 +18,7 @@
 #include "metrics.hh"
 #include "secondary.hh"
 #include <lcdf/error.hh>
+#include <lcdf/straccum.hh>
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -34,10 +35,9 @@ PermString DvipsEncoding::dot_notdef(".notdef");
 #define NEXT_GLYPH_NAME(gn)	("/" + (gn))
 
 int
-DvipsEncoding::parse_glyphlist(String text)
+DvipsEncoding::add_glyphlist(String text)
 {
     // XXX ignores glyph names that map to multiple characters
-    glyphlist.clear();
     const char *data = text.c_str();
     int pos = 0, len = text.length();
     while (1) {
@@ -57,24 +57,24 @@ DvipsEncoding::parse_glyphlist(String text)
 	  read_uni:
 	    if (first == pos
 		|| pos + 1 >= len
-		|| data[pos] != ';'
+		|| (data[pos] != ';' && data[pos] != ',')
 		|| !isxdigit(data[pos+1])
 		|| ((value = strtol(data + pos + 1, &next, 16)),
-		    (!isspace(*next) && *next && *next != ';')))
+		    (!isspace(*next) && *next && *next != ';' && *next != ',')))
 		return -1;
 	    while (*next == ' ' || *next == '\t')
 		next++;
 	    if (*next == '\r' || *next == '\n')
 		glyphlist.insert(glyph_name, value);
-	    else if (*next == ';')
+	    else if (*next == ';' || *next == ',')
 		glyphlist.insert(glyph_name, value | GLYPHLIST_MORE);
 	    else
-		while (*next != '\r' && *next != '\n' && *next != ';')
+		while (*next != '\r' && *next != '\n' && *next != ';' && *next != ',')
 		    next++;
 	    pos = next - data;
-	    if (*next == ';') {	// read another possibility
+	    if (*next == ';' || *next == ',') {	// read another possibility
 		glyph_name = NEXT_GLYPH_NAME(glyph_name);
-		// XXX given "DDDD;EEEE EEEE;FFFF", will not store "FFFF"
+		// XXX given "DDDD,EEEE EEEE,FFFF", will not store "FFFF"
 		goto read_uni;
 	    }
 	} else
@@ -136,7 +136,8 @@ DvipsEncoding::glyphname_unicode(const String &gn, bool *more)
 
 
 DvipsEncoding::DvipsEncoding()
-    : _boundary_char(-1), _altselector_char(-1), _unicoding_map(-1)
+    : _boundary_char(-1), _altselector_char(-1), _unicoding_map(-1),
+      _warn_missing(false)
 {
 }
 
@@ -275,33 +276,36 @@ comment_tokenize(const String &s, int &pos_in, int &line)
     }
 }
 
+static bool
+retokenize_isword(char c)
+{
+    return (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')
+	|| (c >= '0' && c <= '9') || c == '.' || c == '_';
+}
+
 
 static struct { const char *s; int v; } ligkern_ops[] = {
-    { "=:", DvipsEncoding::J_LIG }, { "|=:", DvipsEncoding::J_CLIG },
-    { "|=:>", DvipsEncoding::J_CLIG_S }, { "=:|", DvipsEncoding::J_LIGC },
-    { "=:|>", DvipsEncoding::J_LIGC_S }, { "|=:|", DvipsEncoding::J_CLIGC },
-    { "|=:>", DvipsEncoding::J_CLIGC_S }, { "|=:|>>", DvipsEncoding::J_CLIGC_SS },
-    { "{}", DvipsEncoding::J_NOKERN }, { "{K}", DvipsEncoding::J_NOKERN },
-    { "{L}", DvipsEncoding::J_NOLIG }, { "{LK}", DvipsEncoding::J_NOLIGKERN },
-    { "{KL}", DvipsEncoding::J_NOLIGKERN }, { "{k}", DvipsEncoding::J_NOKERN },
-    { "{l}", DvipsEncoding::J_NOLIG }, { "{lk}", DvipsEncoding::J_NOLIGKERN },
-    { "{kl}", DvipsEncoding::J_NOLIGKERN },
+    { "=:", DvipsEncoding::JL_LIG }, { "|=:", DvipsEncoding::JL_CLIG },
+    { "|=:>", DvipsEncoding::JL_CLIG_S }, { "=:|", DvipsEncoding::JL_LIGC },
+    { "=:|>", DvipsEncoding::JL_LIGC_S }, { "|=:|", DvipsEncoding::JL_CLIGC },
+    { "|=:>", DvipsEncoding::JL_CLIGC_S }, { "|=:|>>", DvipsEncoding::JL_CLIGC_SS },
+    { "{}", DvipsEncoding::JT_KERN }, { "{K}", DvipsEncoding::JT_KERN },
+    { "{L}", DvipsEncoding::JT_LIG }, { "{LK}", DvipsEncoding::JT_NOLIGKERN },
+    { "{KL}", DvipsEncoding::JT_NOLIGKERN }, { "{k}", DvipsEncoding::JT_KERN },
+    { "{l}", DvipsEncoding::JT_LIG }, { "{lk}", DvipsEncoding::JT_NOLIGKERN },
+    { "{kl}", DvipsEncoding::JT_NOLIGKERN },
     // some encodings have @{@} instead of {}
-    { "@{@}", DvipsEncoding::J_NOKERN },
+    { "@{@}", DvipsEncoding::JT_KERN },
     { 0, 0 }
 };
 
-static const char * const nokern_names[] = {
-    "kern removal", "ligature removal", "lig/kern removal"
-};
-
 static int
 find_ligkern_op(const String &s)
 {
     for (int i = 0; ligkern_ops[i].s; i++)
 	if (ligkern_ops[i].s == s)
 	    return ligkern_ops[i].v;
-    return -1;
+    return 0;
 }
 
 inline bool
@@ -310,6 +314,24 @@ operator==(const DvipsEncoding::Ligature& l1, const DvipsEncoding::Ligature& l2)
     return l1.c1 == l2.c1 && l1.c2 == l2.c2;
 }
 
+void
+DvipsEncoding::add_ligkern(const Ligature &l, int override)
+{
+    Ligature *old = std::find(_lig.begin(), _lig.end(), l);
+    if (old == _lig.end())
+	_lig.push_back(l);
+    else {
+	if ((l.join & JT_KERN) && (override > 0 || !(old->join & JT_KERN))) {
+	    old->join |= JT_KERN;
+	    old->k = l.k;
+	}
+	if ((l.join & JT_LIG) && (override > 0 || !(old->join & JT_LIG))) {
+	    old->join = (old->join & JT_KERN) | (l.join & JT_LIGALL);
+	    old->d = l.d;
+	}
+    }
+}
+
 int
 DvipsEncoding::parse_ligkern_words(Vector<String> &v, int override, ErrorHandler *errh)
 {
@@ -318,6 +340,10 @@ DvipsEncoding::parse_ligkern_words(Vector<String> &v, int override, ErrorHandler
     long l;
     char *endptr;
     if (v.size() == 3) {
+	// empty string fails
+	if (!v[0])
+	    return -1;
+	// boundary char setting
 	if (v[0] == "||" && v[1] == "=") {
 	    char *endptr;
 	    if (override > 0 || _boundary_char < 0)
@@ -326,7 +352,9 @@ DvipsEncoding::parse_ligkern_words(Vector<String> &v, int override, ErrorHandler
 		return 0;
 	    else
 		return errh->error("parse error in boundary character assignment");
-	} else if (v[0] == "^^" && v[1] == "=") {
+	}
+	// altselector char setting
+	if (v[0] == "^^" && v[1] == "=") {
 	    char *endptr;
 	    if (override > 0 || _altselector_char < 0)
 		_altselector_char = strtol(v[2].c_str(), &endptr, 10);
@@ -334,31 +362,44 @@ DvipsEncoding::parse_ligkern_words(Vector<String> &v, int override, ErrorHandler
 		return 0;
 	    else
 		return errh->error("parse error in altselector character assignment");
-	} else if ((l = strtol(v[0].c_str(), &endptr, 0)) && endptr == v[0].end() && v[1] == "=") {
+	}
+	// encoding
+	l = strtol(v[0].c_str(), &endptr, 0);
+	if (endptr == v[0].end() && v[1] == "=") {
 	    if (l >= 0 && l < 256) {
 		if (override > 0 || !_e[l])
 		    encode(l, v[2]);
 		return 0;
 	    } else
-		return errh->error("encoding value '%d' out of range", l); 
-	} else if ((op = find_ligkern_op(v[1])) >= J_NOKERN) {
-	    int av = (v[0] == "*" ? J_ALL : encoding_of(v[0]));
-	    if (av < 0)
-		return errh->warning("'%s' has no encoding, ignoring %s", v[0].c_str(), nokern_names[op - J_NOKERN]);
-	    int bv = (v[2] == "*" ? J_ALL : encoding_of(v[2]));
-	    if (bv < 0)
-		return errh->warning("'%s' has no encoding, ignoring %s", v[2].c_str(), nokern_names[op - J_NOKERN]);
-	    Ligature lig = { av, bv, op, 0 };
-	    Ligature *what = std::find(_lig.begin(), _lig.end(), lig);
-	    if (override > 0 && what < _lig.end())
-		*what = lig;
-	    else if (what == _lig.end())
-		_lig.push_back(lig);
-	    return 0;
-	} else
+		return errh->error("encoding value '%d' out of range", l);
+	}
+	
+	// kern operation
+	if (v[1].length() >= 3 && v[1][0] == '{' && v[1].back() == '}') {
+	    String middle = v[1].substring(1, v[1].length() - 2);
+	    l = strtol(middle.c_str(), &endptr, 0);
+	    if (endptr == middle.end()) {
+		op = JT_KERN;
+		goto found_kernop;
+	    }
+	}
+	op = find_ligkern_op(v[1]);
+	if (!op || (op & JT_ADDLIG))
 	    return -1;
-    } else if (v.size() == 4 && (op = find_ligkern_op(v[2])) >= J_LIG
-	       && op <= J_CLIGC_SS) {
+      found_kernop:
+	int av = (v[0] == "*" ? J_ALL : encoding_of(v[0]));
+	if (av < 0)
+	    return errh->warning("'%s' has no encoding, ignoring '%s'", v[0].c_str(), v[1].c_str());
+	int bv = (v[2] == "*" ? J_ALL : encoding_of(v[2]));
+	if (bv < 0)
+	    return errh->warning("'%s' has no encoding, ignoring '%s'", v[2].c_str(), v[1].c_str());
+	if ((op & JT_KERN) && l && (av == J_ALL || bv == J_ALL))
+	    return errh->warning("'%s %s %s' illegal, only {0} works with *", v[0].c_str(), v[1].c_str(), v[2].c_str());
+	Ligature lig = { av, bv, op, l, 0 };
+	add_ligkern(lig, override);
+	return 0;
+
+    } else if (v.size() == 4 && ((op = find_ligkern_op(v[2])) & JT_ADDLIG)) {
 	int av = encoding_of(v[0], override > 0);
 	if (av < 0)
 	    return (override > 0 ? errh->warning("'%s' has no encoding, ignoring ligature", v[0].c_str()) : -1);
@@ -368,15 +409,39 @@ DvipsEncoding::parse_ligkern_words(Vector<String> &v, int override, ErrorHandler
 	int cv = encoding_of(v[3], override > 0);
 	if (cv < 0)
 	    return (override > 0 ? errh->warning("'%s' has no encoding, ignoring ligature", v[3].c_str()) : -1);
-	Ligature lig = { av, bv, op, cv };
-	Ligature *what = std::find(_lig.begin(), _lig.end(), lig);
-	if (override > 0 && what < _lig.end())
-	    *what = lig;
-	else if (what == _lig.end())
-	    _lig.push_back(lig);
+	Ligature lig = { av, bv, op, 0, cv };
+	add_ligkern(lig, override);
 	return 0;
+	
     } else
-	return errh->error("parse error in LIGKERN");
+	return -EPARSE;
+}
+
+int
+DvipsEncoding::parse_position_words(Vector<String> &v, int override, ErrorHandler *errh)
+{
+    if (v.size() != 4)
+	return -EPARSE;
+
+    int c = encoding_of(v[0], override > 0);
+    if (c < 0)
+	return (override > 0 ? errh->warning("'%s' has no encoding, ignoring positioning", v[0].c_str()) : -1);
+    
+    char *endptr;
+    int pdx, pdy, adx;
+    if (!v[1] || !v[2] || !v[3]
+	|| (pdx = strtol(v[1].c_str(), &endptr, 10), *endptr)
+	|| (pdy = strtol(v[2].c_str(), &endptr, 10), *endptr)
+	|| (adx = strtol(v[3].c_str(), &endptr, 10), *endptr))
+	return errh->error("parse error in POSITION");
+
+    Ligature l = { c, pdx, pdy, adx, 0 };
+    Ligature *old = std::find(_pos.begin(), _pos.end(), l);
+    if (old == _pos.end())
+	_pos.push_back(l);
+    else if (override > 0)
+	*old = l;
+    return 0;
 }
 
 int
@@ -384,7 +449,7 @@ DvipsEncoding::parse_unicoding_words(Vector<String> &v, int override, ErrorHandl
 {
     int av;
     if (v.size() < 2 || (v[1] != "=" && v[1] != "=:" && v[1] != ":="))
-	return errh->error("parse error in UNICODING");
+	return -EPARSE;
     else if (v[0] == "||" || (av = encoding_of(v[0])) < 0)
 	return errh->error("target '%s' has no encoding, ignoring UNICODING", v[0].c_str());
 
@@ -417,8 +482,46 @@ DvipsEncoding::parse_unicoding_words(Vector<String> &v, int override, ErrorHandl
     return 0;
 }
 
+const DvipsEncoding::WordType DvipsEncoding::word_types[] = {
+    { "LIGKERN", &DvipsEncoding::parse_ligkern_words },
+    { "POSITION", &DvipsEncoding::parse_position_words },
+    { "UNICODING", &DvipsEncoding::parse_unicoding_words }
+};
+
+void
+DvipsEncoding::parse_word_group(Vector<String> &words, int override, int wt, ErrorHandler *errh)
+{
+    if (words.size() > 0) {
+	int (DvipsEncoding::*method)(Vector<String> &, int, ErrorHandler *) = word_types[wt].parsefunc;
+	if ((this->*method)(words, override, errh) == -EPARSE) {
+	    Vector<String> rewords;
+	    for (String *sp = words.begin(); sp != words.end(); sp++) {
+		const char *s = sp->begin(), *ends = sp->end();
+		while (s != ends) {
+		    const char *word = s;
+		    if (*s == '{') {
+			for (s++; s != ends && *s != '}'; s++)
+			    /* nada */;
+			if (s != ends)
+			    s++;
+		    } else {
+			bool x = retokenize_isword(*s);
+			for (s++; s != ends && x == retokenize_isword(*s); s++)
+			    /* nada */;
+		    }
+		    rewords.push_back(sp->substring(word, s));
+		}
+	    }
+	    if ((this->*method)(rewords, override, errh) == -EPARSE)
+		errh->error("parse error in %s", word_types[wt].name);
+		
+	}
+	words.clear();
+    }
+}
+
 int
-DvipsEncoding::parse_words(const String &s, int override, int (DvipsEncoding::*method)(Vector<String> &, int, ErrorHandler *), ErrorHandler *errh)
+DvipsEncoding::parse_words(const String &s, int override, int wt, ErrorHandler *errh)
 {
     Vector<String> words;
     const char *data = s.data();
@@ -431,15 +534,11 @@ DvipsEncoding::parse_words(const String &s, int override, int (DvipsEncoding::*m
 	    data++;
 	if (data == first) {
 	    data++;		// step past semicolon (or harmlessly past EOS)
-	    if (words.size() > 0) {
-		(this->*method)(words, override, errh);
-		words.clear();
-	    }
+	    parse_word_group(words, override, wt, errh);
 	} else
 	    words.push_back(s.substring(first, data));
     }
-    if (words.size() > 0)
-	(this->*method)(words, override, errh);
+    parse_word_group(words, override, wt, errh);
     return 0;
 }
 
@@ -449,6 +548,22 @@ landmark(const String &filename, int line)
     return filename + String::stable_string(":", 1) + String(line);
 }
 
+static String
+trim_space(const String &s, int pos)
+{
+    while (pos < s.length() && isspace(s[pos]))
+	pos++;
+    int epos = s.length();
+    for (int x = 0; x < 2; x++) {
+	while (epos > pos && isspace(s[epos - 1]))
+	    epos--;
+	if (epos == pos || s[epos - 1] != ';')
+	    break;
+	epos--;
+    }
+    return s.substring(pos, epos - pos);
+}
+
 int
 DvipsEncoding::parse(String filename, bool ignore_ligkern, bool ignore_other, ErrorHandler *errh)
 {
@@ -486,33 +601,34 @@ DvipsEncoding::parse(String filename, bool ignore_ligkern, bool ignore_other, Er
 	    && isspace(token[7])
 	    && !ignore_ligkern) {
 	    lerrh.set_landmark(landmark(filename, line));
-	    parse_words(token.substring(8), 1, &DvipsEncoding::parse_ligkern_words, &lerrh);
+	    parse_words(token.substring(8), 1, WT_LIGKERN, &lerrh);
 	    
 	} else if (token.length() >= 9
 		   && memcmp(token.data(), "LIGKERNX", 8) == 0
 		   && isspace(token[8])
 		   && !ignore_ligkern) {
 	    lerrh.set_landmark(landmark(filename, line));
-	    parse_words(token.substring(9), 1, &DvipsEncoding::parse_ligkern_words, &lerrh);
+	    parse_words(token.substring(9), 1, WT_LIGKERN, &lerrh);
 	    
 	} else if (token.length() >= 10
 		   && memcmp(token.data(), "UNICODING", 9) == 0
 		   && isspace(token[9])
 		   && !ignore_other) {
 	    lerrh.set_landmark(landmark(filename, line));
-	    parse_words(token.substring(10), 1, &DvipsEncoding::parse_unicoding_words, &lerrh);
+	    parse_words(token.substring(10), 1, WT_UNICODING, &lerrh);
+	    
+	} else if (token.length() >= 9
+		   && memcmp(token.data(), "POSITION", 8) == 0
+		   && isspace(token[8])
+		   && !ignore_other) {
+	    lerrh.set_landmark(landmark(filename, line));
+	    parse_words(token.substring(9), 1, WT_POSITION, &lerrh);
 	    
 	} else if (token.length() >= 13
 		   && memcmp(token.data(), "CODINGSCHEME", 12) == 0
 		   && isspace(token[12])
 		   && !ignore_other) {
-	    int p = 13;
-	    while (p < token.length() && isspace(token[p]))
-		p++;
-	    int pp = token.length() - 1;
-	    while (pp > p && isspace(token[pp]))
-		pp--;
-	    _coding_scheme = token.substring(p, pp - p);
+	    _coding_scheme = trim_space(token, 13);
 	    if (_coding_scheme.length() > 39)
 		lerrh.lwarning(landmark(filename, line), "only first 39 chars of CODINGSCHEME are significant");
 	    if (std::find(_coding_scheme.begin(), _coding_scheme.end(), '(') < _coding_scheme.end()
@@ -520,6 +636,18 @@ DvipsEncoding::parse(String filename, bool ignore_ligkern, bool ignore_other, Er
 		lerrh.lerror(landmark(filename, line), "CODINGSCHEME cannot contain parentheses");
 		_coding_scheme = String();
 	    }
+
+	} else if (token.length() >= 11
+		   && memcmp(token.data(), "WARNMISSING", 11) == 0
+		   && (token.length() == 11 || isspace(token[11]))
+		   && !ignore_other) {
+	    String value = trim_space(token, 11);
+	    if (value == "1" || value == "yes" || value == "true" || !value)
+		_warn_missing = true;
+	    else if (value == "0" || value == "no" || value == "false")
+		_warn_missing = false;
+	    else
+		lerrh.lerror(landmark(filename, line), "WARNMISSING command not understood");
 	}
 
     return 0;
@@ -528,22 +656,42 @@ DvipsEncoding::parse(String filename, bool ignore_ligkern, bool ignore_other, Er
 int
 DvipsEncoding::parse_ligkern(const String &ligkern_text, int override, ErrorHandler *errh)
 {
-    return parse_words(ligkern_text, override, &DvipsEncoding::parse_ligkern_words, errh);
+    return parse_words(ligkern_text, override, WT_LIGKERN, errh);
+}
+
+int
+DvipsEncoding::parse_position(const String &position_text, int override, ErrorHandler *errh)
+{
+    return parse_words(position_text, override, WT_POSITION, errh);
 }
 
 int
 DvipsEncoding::parse_unicoding(const String &unicoding_text, int override, ErrorHandler *errh)
 {
-    return parse_words(unicoding_text, override, &DvipsEncoding::parse_unicoding_words, errh);
+    return parse_words(unicoding_text, override, WT_UNICODING, errh);
 }
 
 void
-DvipsEncoding::bad_codepoint(int code)
+DvipsEncoding::bad_codepoint(int code, Metrics &metrics, Vector<String> &unencoded)
 {
     for (int i = 0; i < _lig.size(); i++) {
 	Ligature &l = _lig[i];
-	if (l.c1 == code || l.c2 == code || l.d == code)
-	    l.join = J_BAD;
+	if (l.c1 == code || l.c2 == code)
+	    l.join = 0;
+	else if ((l.join & JT_ADDLIG) && l.d == code)
+	    l.join &= ~JT_LIGALL;
+    }
+
+    if (_warn_missing) {
+	Vector<uint32_t> unicodes;
+	bool unicodes_explicit = x_unicodes(_e[code], unicodes);
+	if (!unicodes_explicit || unicodes.size() > 0) {
+	    Vector<Setting> v;
+	    v.push_back(Setting(Setting::RULE, 500, 500));
+	    v.push_back(Setting(Setting::SPECIAL, String("Warning: missing glyph '") + _e[code] + "'"));
+	    metrics.encode_virtual(code, _e[code], 0, v);
+	    unencoded.push_back(_e[code]);
+	}
     }
 }
 
@@ -650,7 +798,7 @@ DvipsEncoding::make_metrics(Metrics &metrics, const Efont::OpenType::Cmap &cmap,
 	// 1. We were not able to find the glyph using Unicode.
 	// 2. There might be a named_glyph.
 	// May need to try secondaries later.  Store this slot.
-	// Try secondaries, if there is no named_glyph, or explicit unicoding.
+	// Try secondaries, if there's explicit unicoding or no named_glyph.
 	if (unicodes_explicit || named_glyph <= 0)
 	    for (uint32_t *u = unicodes.begin(); u < unicodes.end(); u++)
 		if (secondary->encode_uni(code, chname, *u, metrics, errh))
@@ -671,9 +819,25 @@ DvipsEncoding::make_metrics(Metrics &metrics, const Efont::OpenType::Cmap &cmap,
     }
 
     // final pass: complain
+    Vector<String> unencoded;
     for (int code = 0; code < _e.size(); code++)
 	if (_e[code] != dot_notdef && metrics.glyph(code) <= 0)
-	    bad_codepoint(code);
+	    bad_codepoint(code, metrics, unencoded);
+
+    if (unencoded.size() == 1) {
+	errh->warning("'%s' glyph not found in font", unencoded[0].c_str());
+	errh->message("(This glyph will appear as a blot and cause warnings if used.)");
+    } else if (unencoded.size() > 1) {
+	std::sort(unencoded.begin(), unencoded.end());
+	StringAccum sa;
+	for (const String* a = unencoded.begin(); a < unencoded.end(); a++)
+	    sa << *a << ' ';
+	sa.pop_back();
+	sa.append_break_lines(sa.take_string(), 68, "  ");
+	sa.pop_back();
+	errh->warning("%d glyphs not found in font:", unencoded.size());
+	errh->message("%s\n(These glyphs will appear as blots and cause warnings if used.)", sa.c_str());
+    }
 
     metrics.set_coding_scheme(_coding_scheme);
 }
@@ -683,22 +847,19 @@ void
 DvipsEncoding::apply_ligkern_lig(Metrics &metrics, ErrorHandler *errh) const
 {
     assert((int)J_ALL == (int)Metrics::CODE_ALL);
-    for (int i = 0; i < _lig.size(); i++) {
-	const Ligature &l = _lig[i];
-	if (l.c1 < 0 || l.c2 < 0 || l.join < 0 || l.join == J_NOKERN)
+    for (const Ligature *l = _lig.begin(); l < _lig.end(); l++) {
+	if (l->c1 < 0 || l->c2 < 0 || l->join < 0 || !(l->join & JT_LIG))
+	    continue;
+	metrics.remove_ligatures(l->c1, l->c2);
+	if (!(l->join & JT_ADDLIG))
 	    /* nada */;
-	else if (l.join == J_NOLIG || l.join == J_NOLIGKERN)
-	    metrics.remove_ligatures(l.c1, l.c2);
-	else if (l.join == J_LIG) {
-	    metrics.remove_ligatures(l.c1, l.c2);
-	    metrics.add_ligature(l.c1, l.c2, l.d);
-	} else if (l.join == J_LIGC) {
-	    metrics.remove_ligatures(l.c1, l.c2);
-	    metrics.add_ligature(l.c1, l.c2, metrics.pair_code(l.d, l.c2));
-	} else if (l.join == J_CLIG) {
-	    metrics.remove_ligatures(l.c1, l.c2);
-	    metrics.add_ligature(l.c1, l.c2, metrics.pair_code(l.c1, l.d));
-	} else {
+	else if ((l->join & JT_LIGALL) == JL_LIG)
+	    metrics.add_ligature(l->c1, l->c2, l->d);
+	else if ((l->join & JT_LIGALL) == JL_LIGC)
+	    metrics.add_ligature(l->c1, l->c2, metrics.pair_code(l->d, l->c2));
+	else if ((l->join & JT_LIGALL) == JL_CLIG)
+	    metrics.add_ligature(l->c1, l->c2, metrics.pair_code(l->c1, l->d));
+	else {
 	    static int complex_join_warning = 0;
 	    if (!complex_join_warning) {
 		errh->warning("complex LIGKERN ligature removed (I only support '=:', '=:|', and '|=:')");
@@ -712,10 +873,15 @@ void
 DvipsEncoding::apply_ligkern_kern(Metrics &metrics, ErrorHandler *) const
 {
     assert((int)J_ALL == (int)Metrics::CODE_ALL);
-    for (int i = 0; i < _lig.size(); i++) {
-	const Ligature &l = _lig[i];
-	if (l.c1 >= 0 && l.c2 >= 0
-	    && (l.join == J_NOKERN || l.join == J_NOLIGKERN))
-	    metrics.remove_kerns(l.c1, l.c2);
-    }
+    for (const Ligature *l = _lig.begin(); l < _lig.end(); l++)
+	if (l->c1 >= 0 && l->c2 >= 0 && (l->join & JT_KERN))
+	    metrics.set_kern(l->c1, l->c2, l->k);
+}
+
+void
+DvipsEncoding::apply_position(Metrics &metrics, ErrorHandler *) const
+{
+    for (const Ligature *l = _pos.begin(); l < _pos.end(); l++)
+	if (l->c1 >= 0)
+	    metrics.add_single_positioning(l->c1, l->c2, l->join, l->k);
 }
