@@ -1,6 +1,6 @@
 /* metrics.{cc,hh} -- an encoding during and after OpenType features
  *
- * Copyright (c) 2003-2004 Eddie Kohler
+ * Copyright (c) 2003-2005 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -205,7 +205,7 @@ Metrics::base_glyphs(Vector<Glyph> &v) const
     v.assign(_encoding.size(), 0);
     for (Code c = 0; c < _encoding.size(); c++)
 	if (_encoding[c].base_code >= 0)
-	    v[c] = _encoding[ _encoding[c].base_code ].glyph;
+	    v[_encoding[c].base_code] = _encoding[c].glyph;
 }
 
 
@@ -216,10 +216,15 @@ void
 Metrics::Char::clear()
 {
     glyph = 0;
+    base_code = -1;
+    unicode = 0;
     ligatures.clear();
+    kerns.clear();
     delete virtual_char;
     virtual_char = 0;
     pdx = pdy = adx = 0;
+    built_in1 = built_in2 = -1;
+    lookup_source = -1;
     flags = 0;
 }
 
@@ -227,20 +232,21 @@ void
 Metrics::Char::swap(Char &c)
 {
     std::swap(glyph, c.glyph);
+    // NB: only a partial switch of base_code!!
+    if (base_code < 0)
+	base_code = c.base_code;
+    c.base_code = -1;
+    std::swap(unicode, c.unicode);
     ligatures.swap(c.ligatures);
     kerns.swap(c.kerns);
     std::swap(virtual_char, c.virtual_char);
     std::swap(pdx, c.pdx);
     std::swap(pdy, c.pdy);
     std::swap(adx, c.adx);
-    std::swap(flags, c.flags);
     std::swap(built_in1, c.built_in1);
     std::swap(built_in2, c.built_in2);
     std::swap(lookup_source, c.lookup_source);
-    // NB: only a partial switch of base_code!!
-    if (base_code < 0)
-	base_code = c.base_code;
-    c.base_code = -1;
+    std::swap(flags, c.flags);
 }
 
 
@@ -364,19 +370,24 @@ Metrics::add_kern(Code in1, Code in2, int kern)
 }
 
 void
-Metrics::remove_kerns(Code in1, Code in2)
+Metrics::set_kern(Code in1, Code in2, int kern)
 {
     if (in1 == CODE_ALL) {
 	for (in1 = 0; in1 < _encoding.size(); in1++)
-	    remove_kerns(in1, in2);
+	    set_kern(in1, in2, kern);
     } else {
 	Char &ch = _encoding[in1];
-	if (in2 == CODE_ALL)
+	if (in2 == CODE_ALL) {
+	    assert(kern == 0);
 	    ch.kerns.clear();
-	else if (Kern *k = kern_obj(in1, in2)) {
-	    *k = ch.kerns.back();
-	    ch.kerns.pop_back();
-	}
+	} else if (Kern *k = kern_obj(in1, in2)) {
+	    if (kern == 0) {
+		*k = ch.kerns.back();
+		ch.kerns.pop_back();
+	    } else
+		k->kern = kern;
+	} else if (kern != 0)
+	    ch.kerns.push_back(Kern(in2, kern));
     }
 }
 
@@ -960,17 +971,19 @@ Metrics::cut_encoding(int size)
     /* Remove ligatures and kerns that point beyond 'size', except for valid
        context ligatures. Also remove ligatures that have non-live
        components. */
+    /* 30.May.2005 -- Kerns might point involve a too-high character; kill
+       them. */
     for (Code c = 0; c < size; c++) {
 	Char &ch = _encoding[c];
 	for (Ligature *l = ch.ligatures.begin(); l != ch.ligatures.end(); l++)
-	    if (!good[l->in2]
+	    if (!good[l->in2] || l->in2 >= size
 		|| (!good[l->out] && !_encoding[l->out].context_setting(c, l->in2))) {
 		*l = ch.ligatures.back();
 		ch.ligatures.pop_back();
 		l--;
 	    }
 	for (Kern *k = ch.kerns.begin(); k != ch.kerns.end(); k++)
-	    if (!good[k->in2]) {
+	    if (!good[k->in2] || k->in2 >= size) {
 		*k = ch.kerns.back();
 		ch.kerns.pop_back();
 		k--;
@@ -981,7 +994,6 @@ Metrics::cut_encoding(int size)
 }
 
 namespace {
-
 // preference-sorting extra characters
 enum { BASIC_LATIN_SCORE = 2, LATIN1_SUPPLEMENT_SCORE = 5,
        LOW_16_SCORE = 6, OTHER_SCORE = 7, NOCHAR_SCORE = 100000,
@@ -1109,7 +1121,7 @@ Metrics::shrink_encoding(int size, const DvipsEncoding &dvipsenc, ErrorHandler *
 	    }
 	}
 
-    /* List empty slots in a two phases: Those not encoded by the input
+    /* List empty slots in two phases: Those not encoded by the input
        encoding, then those encoded by the input encoding (but that character
        wasn't available). */
     Vector<Code> empty_codes;
@@ -1121,34 +1133,28 @@ Metrics::shrink_encoding(int size, const DvipsEncoding &dvipsenc, ErrorHandler *
 
     /* Then, loop over the unencoded characters, assigning them codes. */
     int nunencoded = 0;
-    Code *both_hole, *overlay_hole, *base_hole, *end_hole;
-    both_hole = overlay_hole = base_hole = empty_codes.begin();
+    Code *holes[4], *end_hole;
+    holes[1] = holes[2] = holes[3] = empty_codes.begin();
     end_hole = empty_codes.end();
 
     for (Slot *slot = slots.begin(); slot != slots.end(); slot++) {
 	if (slot->new_code >= 0)
 	    continue;
-	
-	bool need_base = _encoding[slot->old_code].visible_base();
-	bool need_overlay = _encoding[slot->old_code].flag(Char::LIVE);
+
+	int needs = (_encoding[slot->old_code].visible_base() ? 1 : 0)
+	    + (_encoding[slot->old_code].flag(Char::LIVE) ? 2 : 0);
+	assert(needs > 0);
 	Code **hole;
-	assert(need_base || need_overlay);
-	if (need_base && need_overlay) {
-	    while (both_hole < end_hole && (_encoding[*both_hole].visible() || _encoding[*both_hole].base_code >= 0))
-		both_hole++;
-	    hole = &both_hole;
-	} else if (need_base) {
-	    while (base_hole < end_hole && _encoding[*base_hole].base_code >= 0)
-		base_hole++;
-	    hole = &base_hole;
-	} else {
-	    while (overlay_hole < end_hole && _encoding[*overlay_hole].visible())
-		overlay_hole++;
-	    hole = &overlay_hole;
+
+	for (hole = &holes[needs]; *hole < end_hole; (*hole)++) {
+	    int haves = (!_encoding[**hole].base_code >= 0 ? 1 : 0)
+		+ (!_encoding[**hole].visible() ? 2 : 0);
+	    if ((needs & haves) == needs)
+		break;
 	}
 
 	if (*hole < end_hole) {
-	    if (need_overlay) {
+	    if (needs & 2) {
 		assert(!_encoding[**hole].visible());
 		_encoding[**hole].swap(_encoding[slot->old_code]);
 		slot->new_code = **hole;
@@ -1156,7 +1162,7 @@ Metrics::shrink_encoding(int size, const DvipsEncoding &dvipsenc, ErrorHandler *
 		_encoding[slot->old_code].base_code = **hole;
 		slot->new_code = slot->old_code;
 	    }
-	    if (need_base) {
+	    if (needs & 1) {
 		assert(_encoding[**hole].base_code < 0 || _encoding[**hole].base_code == slot->old_code);
 		_encoding[**hole].base_code = slot->old_code;
 	    }
@@ -1200,11 +1206,10 @@ font, so I've ignored these:\n%s.)", sa.c_str());
 void
 Metrics::make_base(int size)
 {
-    if (_encoding.size() <= size) {
+    if (_encoding.size() <= size)
 	/* If the encoding has not grown, there cannot be any base characters
 	   left to swap in. */
 	return;
-    }
     
     assert(_encoding.size() > size);
     bool reencoded = false;
@@ -1219,10 +1224,8 @@ Metrics::make_base(int size)
 	    _encoding[c].swap(_encoding[ch.base_code]);
 	    reencoded = true;
 	}
-	if (ch.virtual_char) {	// force it to be removed by cut_encoding
-	    ch.virtual_char->setting[0] = Setting(Setting::SHOW, size + 1);
-	    reencoded = true;
-	}
+	if (ch.virtual_char)	// remove it
+	    ch.clear();
     }
     if (reencoded) {
 	reencode(reencoding);
@@ -1264,6 +1267,9 @@ Metrics::setting(Code code, Vector<Setting> &v, SettingMode sm) const
 	    switch (s->op) {
 	      case Setting::MOVE:
 	      case Setting::RULE:
+	      case Setting::PUSH:
+	      case Setting::POP:
+	      case Setting::SPECIAL:
 		v.push_back(*s);
 		break;
 	      case Setting::FONT:
@@ -1293,7 +1299,7 @@ Metrics::setting(Code code, Vector<Setting> &v, SettingMode sm) const
 	if (ch.pdx != 0 || ch.pdy != 0)
 	    v.push_back(Setting(Setting::MOVE, ch.pdx, ch.pdy));
 	
-	v.push_back(Setting(Setting::SHOW, ch.base_code, base_glyph(code)));
+	v.push_back(Setting(Setting::SHOW, ch.base_code, ch.glyph));
 	
 	if (ch.pdy != 0 || ch.adx - ch.pdx != 0)
 	    v.push_back(Setting(Setting::MOVE, ch.adx - ch.pdx, -ch.pdy));
@@ -1386,6 +1392,15 @@ Metrics::unparse() const
 		      case Setting::RULE:
 			fprintf(stderr, " [%d,%d]", s->x, s->y);
 			break;
+		      case Setting::PUSH:
+			fprintf(stderr, " (");
+			break;
+		      case Setting::POP:
+			fprintf(stderr, " )");
+			break;
+		      case Setting::SPECIAL:
+			fprintf(stderr, " S{%s}", s->s.c_str());
+			break;
 		    }
 		fprintf(stderr, "  ((%d/%s, %d/%s))\n", ch.built_in1, code_str(ch.built_in1), ch.built_in2, code_str(ch.built_in2));
 	    }
