@@ -94,6 +94,8 @@ using namespace Efont;
 #define ALTERNATES_FILTER_OPT	336
 #define SPACE_FACTOR_OPT	337
 #define MATH_SPACING_OPT	338
+#define POSITION_OPT		339
+#define WARN_MISSING_OPT	340
 
 #define AUTOMATIC_OPT		341
 #define FONT_NAME_OPT		342
@@ -148,6 +150,8 @@ static Clp_Option options[] = {
     { "minimum-kern", 'k', MINIMUM_KERN_OPT, Clp_ArgDouble, 0 },
     { "kern-precision", 'k', MINIMUM_KERN_OPT, Clp_ArgDouble, 0 },
     { "ligkern", 0, LIGKERN_OPT, Clp_ArgString, 0 },
+    { "position", 0, POSITION_OPT, Clp_ArgString, 0 },
+    { "warn-missing", 0, WARN_MISSING_OPT, 0, Clp_Negate },
     { "no-encoding-commands", 0, NO_ECOMMAND_OPT, 0, Clp_OnlyNegated },
     { "default-ligkern", 0, DEFAULT_LIGKERN_OPT, 0, Clp_Negate },
     { "unicoding", 0, UNICODING_OPT, Clp_ArgString, 0 },
@@ -162,8 +166,8 @@ static Clp_Option options[] = {
     { "clear-alternates", 0, CLEAR_ALTERNATES_OPT, 0, 0 },
     { "alternates-filter", 0, ALTERNATES_FILTER_OPT, Clp_ArgString, 0 },
     { "space-factor", 0, SPACE_FACTOR_OPT, Clp_ArgDouble, 0 },
-    { "math-spacing", 0, MATH_SPACING_OPT, 0, Clp_Negate },
-    
+    { "math-spacing", 0, MATH_SPACING_OPT, CHAR_OPTTYPE, Clp_Negate | Clp_Optional },
+
     { "pl", 'p', PL_OPT, 0, 0 },
     { "virtual", 0, VIRTUAL_OPT, 0, Clp_Negate },
     { "no-encoding", 0, NO_ENCODING_OPT, 0, Clp_OnlyNegated },
@@ -172,7 +176,7 @@ static Clp_Option options[] = {
     { "no-updmap", 0, NO_UPDMAP_OPT, 0, Clp_OnlyNegated },
     { "map-file", 0, MAP_FILE_OPT, Clp_ArgString, Clp_Negate },
     { "output-encoding", 0, OUTPUT_ENCODING_OPT, Clp_ArgString, Clp_Optional },
-        
+    
     { "automatic", 'a', AUTOMATIC_OPT, 0, Clp_Negate },
     { "name", 'n', FONT_NAME_OPT, Clp_ArgString, 0 },
     { "vendor", 'v', VENDOR_OPT, Clp_ArgString, 0 },
@@ -235,6 +239,7 @@ static double design_size;
 static double minimum_kern = 2.0;
 static double space_factor = 1.0;
 static bool math_spacing = false;
+static int skew_char = -1;
 
 static String out_encoding_file;
 static String out_encoding_name;
@@ -288,7 +293,7 @@ Font feature and transformation options:\n\
   -E, --extend=F               Widen characters by a factor of F.\n\
   -S, --slant=AMT              Oblique characters by AMT, generally <<1.\n\
   -L, --letterspacing=AMT      Letterspace each character by AMT units.\n\
-      --math-spacing           Use letterspacing appropriate for math.\n\
+      --math-spacing[=SKEWCH]  Use letterspacing appropriate for math.\n\
   -k, --min-kern=N             Omit kerns with absolute value < N [2.0].\n\
       --space-factor=F         Scale wordspace by a factor of F.\n\
       --design-size=SIZE       Set font design size to SIZE.\n\
@@ -296,19 +301,21 @@ Font feature and transformation options:\n\
     printf("\
 Encoding options:\n\
   -e, --encoding=FILE          Use DVIPS encoding FILE as a base encoding.\n\
-      --literal-encoding=FILE  Use DVIPS encoding FILE verbatim.\n\
-      --ligkern=COMMAND        Add a LIGKERN command.\n\
-      --unicoding=COMMAND      Add a UNICODING command.\n\
-      --no-encoding-commands   Ignore encoding file's LIGKERN/UNICODINGs.\n\
-      --default-ligkern        Ignore encoding file's LIGKERNs, use defaults.\n\
-      --coding-scheme=SCHEME   Set the output coding scheme to SCHEME.\n\
       --boundary-char=CHAR     Set the boundary character to CHAR.\n\
       --altselector-char=CHAR  Set the alternate selector character to CHAR.\n\
       --altselector-feature=F  Activate feature F for --altselector-char.\n\
       --alternates-filter=PAT  Include only alternate characters matching PAT.\n\
-      --include-alternates=PAT Same, but cumulative.\n\
+      --include-alternates=PAT Same as --alternates-filter, but cumulative.\n\
       --exclude-alternates=PAT Ignore alternate characters matching PAT.\n\
       --clear-alternates       Clear included/excluded alternates.\n\
+      --ligkern=COMMAND        Add a ligature or kern.\n\
+      --position=COMMAND       Add a POSITION command.\n\
+      --unicoding=COMMAND      Add a UNICODING command.\n\
+      --no-encoding-commands   Ignore encoding file's LIGKERN/UNICODINGs.\n\
+      --no-default-ligkern     Don't include default LIGKERNs.\n\
+      --coding-scheme=SCHEME   Set the output coding scheme to SCHEME.\n\
+      --warn-missing           Warn about characters not supported by font.\n\
+      --literal-encoding=FILE  Use DVIPS encoding FILE verbatim.\n\
 \n");
     printf("\
 Automatic mode options:\n\
@@ -406,14 +413,14 @@ make_base_font_name(const String &font_name)
 }
 
 static double
-get_design_size(const OpenType::Font &otf)
+get_design_size(const FontInfo &finfo)
 {
     try {
-	String gpos_table = otf.table("GPOS");
+	String gpos_table = finfo.otf->table("GPOS");
 	if (!gpos_table)
 	    throw OpenType::Error();
 
-	ErrorHandler *errh = ErrorHandler::silent_handler();
+	ErrorHandler *errh = ErrorHandler::ignore_handler();
 	OpenType::Gpos gpos(gpos_table, errh);
 
 	// extract 'size' feature(s)
@@ -476,10 +483,17 @@ real_string(double value, double du)
     }
 }
 
+double
+font_slant(const FontInfo &finfo)
+{
+    double val;
+    (void) finfo.cff->dict_value(Efont::Cff::oItalicAngle, &val);
+    return -tan(val * M_PI / 180.0) + slant;
+}
+
 static void
 output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
-	  const OpenType::Font &family_otf, const Cff::Font *family_cff,
-	  bool vpl, FILE *f)
+	  const FontInfo &finfo, bool vpl, FILE *f)
 {
     // XXX check DESIGNSIZE and DESIGNUNITS for correctness
 
@@ -496,7 +510,7 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
     int design_units = metrics.design_units();
 
     if (design_size <= 0)
-	design_size = get_design_size(family_otf);
+	design_size = get_design_size(finfo);
     fprintf(f, "(DESIGNSIZE R %.1f)\n"
 	    "(DESIGNUNITS R %d.0)\n"
 	    "(COMMENT DESIGNSIZE (1 em) IS IN POINTS)\n"
@@ -512,19 +526,16 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
     int bounds[4], width;
     double du = (design_units == 1000 ? 1. : design_units / 1000.);
     
-    double val;
-    (void) family_cff->dict_value(Efont::Cff::oItalicAngle, &val);
-    double actual_slant = -tan(val * M_PI / 180.0) + slant;
+    double actual_slant = font_slant(finfo);
     if (actual_slant)
 	fprintf(f, "   (SLANT R %g)\n", actual_slant);
 
-    OpenType::Cmap cmap(family_otf.table("cmap"));
-    
-    if (char_bounds(bounds, width, family_cff, cmap, ' ', font_xform)) {
+    double val;
+    if (char_bounds(bounds, width, finfo, font_xform, ' ')) {
 	// advance space width by letterspacing, scale by space_factor
 	double space_width = (width + letterspace) * space_factor;
 	fprint_real(f, "   (SPACE", space_width, du);
-	if (family_cff->dict_value(Efont::Cff::oIsFixedPitch, &val) && val) {
+	if (finfo.cff->dict_value(Efont::Cff::oIsFixedPitch, &val) && val) {
 	    // fixed-pitch: no space stretch or shrink
 	    fprint_real(f, "   (STRETCH", 0, du);
 	    fprint_real(f, "   (SHRINK", 0, du);
@@ -536,12 +547,7 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
 	}
     }
 
-    // XXX what if 'x', 'm', 'z' were subject to substitution?
-    int xheight = 1000;
-    static const int xheight_unis[] = { 'x', 'm', 'z', 0 };
-    for (const int *x = xheight_unis; *x; x++)
-	if (char_bounds(bounds, width, family_cff, cmap, 'x', font_xform) && bounds[3] < xheight)
-	    xheight = bounds[3];
+    int xheight = font_x_height(finfo, font_xform);
     if (xheight < 1000)
 	fprint_real(f, "   (XHEIGHT", xheight, du);
     
@@ -564,32 +570,35 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
     Vector<String> glyph_ids;
     Vector<String> glyph_comments(257, String());
     Vector<String> glyph_base_comments(257, String());
-    for (int i = 0; i < 256; i++) {
+    for (int i = 0; i < metrics.encoding_size(); i++) {
 	if (metrics.glyph(i)) {
 	    PermString name = metrics.code_name(i), expected_name;
 	    if (i >= '0' && i <= '9')
 		expected_name = digit_names[i - '0'];
 	    else if ((i >= 'A' && i <= 'Z') || (i >= 'a' && i <= 'z'))
 		expected_name = PermString((char)i);
-	    
+	    String glyph_comment;
+	    if (name != expected_name)
+		glyph_comment = " (COMMENT " + String(name) + ")";
+
+	    int base = metrics.base_code(i);
+	    if (base >= 0 && base < 256)
+		glyph_base_comments[base] = glyph_comment;
+
+	    if (i >= 256)
+		continue;
+
 	    if (expected_name
 		&& name.length() >= expected_name.length()
 		&& memcmp(name.c_str(), expected_name.c_str(), expected_name.length()) == 0)
 		glyph_ids.push_back("C " + String((char)i));
 	    else
 		glyph_ids.push_back("D " + String(i));
-	    
-	    if (name != expected_name)
-		glyph_comments[i] = " (COMMENT " + String(name) + ")";
 
-	    int base = metrics.base_code(i);
-	    if (base != i)
-		glyph_base_comments[i] = " (COMMENT " + String(metrics.code_name(base)) + ")";
-	    else
-		glyph_base_comments[i] = glyph_comments[i];
+	    glyph_comments[i] = glyph_base_comments[i] = glyph_comment;
 	    
-	} else
-	    glyph_ids.push_back("X");
+	} else if (i < 256)
+	    glyph_ids.push_back("D " + String(i));
     }
     // finally, BOUNDARYCHAR
     glyph_ids.push_back("BOUNDARYCHAR");
@@ -631,6 +640,7 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
     // CHARACTERs
     Vector<Setting> settings;
     StringAccum sa;
+    Vector<Point> push_stack;
     
     for (int i = 0; i < 256; i++)
 	if (metrics.setting(i, settings)) {
@@ -638,34 +648,77 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
 
 	    // unparse settings into DVI commands
 	    sa.clear();
+	    push_stack.clear();
 	    CharstringBounds boundser(font_xform);
-	    const CharstringProgram *program = family_cff;
-	    for (int j = 0; j < settings.size(); j++) {
-		Setting &s = settings[j];
-		if (s.op == Setting::SHOW) {
-		    boundser.char_bounds(program->glyph_context(s.y));
+	    const CharstringProgram *program = finfo.cff;
+	    for (const Setting *s = settings.begin(); s < settings.end(); s++)
+		switch (s->op) {
+		    
+		  case Setting::SHOW:
+		    boundser.char_bounds(program->glyph_context(s->y));
 		    // 3.Aug.2004 -- reported by Marco Kuhlmann: Don't use
 		    // glyph_ids[] array when looking at a different font.
-		    if (program == family_cff)
-			sa << "      (SETCHAR " << glyph_ids[s.x] << ')' << glyph_base_comments[s.x] << "\n";
+		    if (program == finfo.cff)
+			sa << "      (SETCHAR " << glyph_ids[s->x] << ')' << glyph_base_comments[s->x] << "\n";
 		    else
-			sa << "      (SETCHAR D " << s.x << ")\n";
-		} else if (s.op == Setting::MOVE && vpl) {
-		    boundser.translate(s.x, s.y);
-		    if (s.x)
-			sa << "      (MOVERIGHT R " << real_string(s.x, du) << ")\n";
-		    if (s.y)
-			sa << "      (MOVEUP R " << real_string(s.y, du) << ")\n";
-		} else if (s.op == Setting::RULE && vpl) {
+			sa << "      (SETCHAR D " << s->x << ")\n";
+		    break;
+
+		  case Setting::MOVE:
+		    boundser.translate(s->x, s->y);
+		    if (s->x)
+			sa << "      (MOVERIGHT R " << real_string(s->x, du) << ")\n";
+		    if (s->y)
+			sa << "      (MOVEUP R " << real_string(s->y, du) << ")\n";
+		    break;
+
+		  case Setting::RULE:
 		    boundser.mark(Point(0, 0));
-		    boundser.mark(Point(s.x, s.y));
-		    boundser.translate(s.x, 0);
-		    sa << "      (SETRULE R " << real_string(s.y, du) << " R " << real_string(s.x, du) << ")\n";
-		} else if (s.op == Setting::FONT && vpl) {
-		    sa << "      (SELECTFONT D " << (int) s.x << ")\n";
-		    program = metrics.mapped_font((int) s.x);
+		    boundser.mark(Point(s->x, s->y));
+		    boundser.translate(s->x, 0);
+		    sa << "      (SETRULE R " << real_string(s->y, du) << " R " << real_string(s->x, du) << ")\n";
+		    break;
+
+		  case Setting::FONT:
+		    program = metrics.mapped_font((int) s->x);
+		    sa << "      (SELECTFONT D " << (int) s->x << ")\n";
+		    break;
+
+		  case Setting::PUSH:
+		    push_stack.push_back(boundser.transform(Point(0, 0)));
+		    sa << "      (PUSH)\n";
+		    break;
+
+		  case Setting::POP: {
+		      assert(push_stack.size());
+		      Point p = push_stack.back() - boundser.transform(Point(0, 0));
+		      boundser.translate(p.x, p.y);
+		      push_stack.pop_back();
+		      sa << "      (POP)\n";
+		      break;
+		  }
+
+		  case Setting::SPECIAL: {
+		      bool needhex = false;
+		      for (const char *str = s->s.begin(); str < s->s.end() && !needhex; str++)
+			  if (*str < ' ' || *str > '~' || *str == '(' || *str == ')')
+			      needhex = true;
+		      if (needhex) {
+			  sa << "      (SPECIALHEX ";
+			  for (const char *str = s->s.begin(); str < s->s.end(); str++) {
+			      static const char hexdig[] = "0123456789ABCDEF";
+			      int val = (unsigned char) *str;
+			      sa << hexdig[val >> 4] << hexdig[val & 0xF];
+			  }
+			  sa << ")\n";
+		      } else
+			  sa << "      (SPECIAL " << s->s << ")\n";
+		      break;
+		  }
+		    
 		}
-	    }
+	    
+	    assert(push_stack.size() == 0);
 
 	    // output information
 	    boundser.output(bounds, width);
@@ -684,8 +737,7 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
 
 static void
 output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
-	  const OpenType::Font &family_otf, const Cff::Font *family_cff,
-	  bool vpl, String filename, ErrorHandler *errh)
+	  const FontInfo &finfo, bool vpl, String filename, ErrorHandler *errh)
 {
     if (no_create)
 	errh->message("would create %s", filename.c_str());
@@ -693,7 +745,7 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
 	if (verbose)
 	    errh->message("creating %s", filename.c_str());
 	if (FILE *f = fopen(filename.c_str(), "w")) {
-	    output_pl(metrics, ps_name, boundary_char, family_otf, family_cff, vpl, f);
+	    output_pl(metrics, ps_name, boundary_char, finfo, vpl, f);
 	    fclose(f);
 	} else
 	    errh->error("%s: %s", filename.c_str(), strerror(errno));
@@ -859,7 +911,8 @@ output_encoding(const Metrics &metrics,
 	    sa << (i ? "\n%" : "%") << hex_digits[(i >> 4) & 0xF] << '0' << '\n' << ' ';
 	else if ((i & 0x7) == 0)
 	    sa << '\n' << ' ';
-	if (int g = glyphs[i])
+	int g = glyphs[i];
+	if (g > 0 && g < glyph_names.size())
 	    sa << ' ' << '/' << glyph_names[g];
 	else
 	    sa << " /.notdef";
@@ -932,8 +985,8 @@ output_encoding(const Metrics &metrics,
 
 static void
 output_tfm(const Metrics &metrics, const String &ps_name, int boundary_char,
-	   const OpenType::Font &family_otf, const Cff::Font *family_cff,
-	   String tfm_filename, String vf_filename, ErrorHandler *errh)
+	   const FontInfo &finfo, String tfm_filename, String vf_filename,
+	   ErrorHandler *errh)
 {
     String pl_filename;
     int pl_fd = temporary_file(pl_filename, errh);
@@ -949,7 +1002,7 @@ output_tfm(const Metrics &metrics, const String &ps_name, int boundary_char,
 	if (verbose)
 	    errh->message("writing %s to temporary file", (vpl ? "VPL" : "PL"));
 	FILE *f = fdopen(pl_fd, "w");
-	output_pl(metrics, ps_name, boundary_char, family_otf, family_cff, vpl, f);
+	output_pl(metrics, ps_name, boundary_char, finfo, vpl, f);
 	fclose(f);
     }
 
@@ -975,7 +1028,7 @@ output_tfm(const Metrics &metrics, const String &ps_name, int boundary_char,
 
 void
 output_metrics(Metrics &metrics, const String &ps_name, int boundary_char,
-	       const OpenType::Font &family_otf, const Cff::Font *family_cff,
+	       const FontInfo &finfo,
 	       const String &encoding_name, const String &encoding_file,
 	       const String &font_name,
 	       void (*dvips_include)(const String &ps_name, StringAccum &, ErrorHandler *),
@@ -999,19 +1052,21 @@ output_metrics(Metrics &metrics, const String &ps_name, int boundary_char,
 	    String vf = getodir(O_VF, errh) + "/" + font_name + ".vf";
 	    if (no_create)
 		errh->message("would remove potential VF file '%s'", vf.c_str());
-	    else if (verbose)
-		errh->message("removing potential VF file '%s'", vf.c_str());
-	    if (!no_create && unlink(vf.c_str()) < 0 && errno != ENOENT)
-		errh->error("removing %s: %s", vf.c_str(), strerror(errno));
+	    else {
+		if (verbose)
+		    errh->message("removing potential VF file '%s'", vf.c_str());
+		if (unlink(vf.c_str()) < 0 && errno != ENOENT)
+		    errh->error("removing %s: %s", vf.c_str(), strerror(errno));
+	    }
 	}
     } else {
 	if (output_flags & G_BINARY) {
 	    String tfm = getodir(O_TFM, errh) + "/" + font_name + ".tfm";
 	    String vf = getodir(O_VF, errh) + "/" + font_name + ".vf";
-	    output_tfm(metrics, ps_name, boundary_char, family_otf, family_cff, tfm, vf, errh);
+	    output_tfm(metrics, ps_name, boundary_char, finfo, tfm, vf, errh);
 	} else {
 	    String outfile = getodir(O_VPL, errh) + "/" + font_name + ".vpl";
-	    output_pl(metrics, ps_name, boundary_char, family_otf, family_cff, true, outfile, errh);
+	    output_pl(metrics, ps_name, boundary_char, finfo, true, outfile, errh);
 	    update_odir(O_VPL, outfile, errh);
 	}
 	metrics.make_base(257);
@@ -1025,10 +1080,10 @@ output_metrics(Metrics &metrics, const String &ps_name, int boundary_char,
 	/* do nothing */;
     else if (output_flags & G_BINARY) {
 	String tfm = getodir(O_TFM, errh) + "/" + base_font_name + ".tfm";
-	output_tfm(metrics, ps_name, boundary_char, family_otf, family_cff, tfm, String(), errh);
+	output_tfm(metrics, ps_name, boundary_char, finfo, tfm, String(), errh);
     } else {
 	String outfile = getodir(O_PL, errh) + "/" + base_font_name + ".pl";
-	output_pl(metrics, ps_name, boundary_char, family_otf, family_cff, false, outfile, errh);
+	output_pl(metrics, ps_name, boundary_char, finfo, false, outfile, errh);
 	update_odir(O_PL, outfile, errh);
     }
     minimum_kern = save_minimum_kern;
@@ -1185,7 +1240,7 @@ do_gsub(Metrics& metrics, const OpenType::Font& otf, DvipsEncoding& dvipsenc, bo
 	altselector_features.swap(interesting_features);
 	altselector_feature_filters.swap(feature_filters);
 	Vector<Lookup> alt_lookups(gsub.nlookups(), Lookup());
-	find_lookups(gsub.script_list(), gsub.feature_list(), alt_lookups, ErrorHandler::silent_handler());
+	find_lookups(gsub.script_list(), gsub.feature_list(), alt_lookups, ErrorHandler::ignore_handler());
 	Vector<OpenType::Substitution> alt_subs;
 	for (int i = 0; i < alt_lookups.size(); i++)
 	    if (alt_lookups[i].used) {
@@ -1220,6 +1275,41 @@ do_gpos(Metrics& metrics, const OpenType::Font& otf, HashMap<uint32_t, int>& fea
 	}
 }
 
+static void
+do_math_spacing(Metrics &metrics, const FontInfo &finfo,
+		const DvipsEncoding &dvipsenc)
+{
+    Transform font_xform;
+    if (extend)
+	font_xform.scale(extend, 1);
+    if (slant)
+	font_xform.shear(slant);
+    CharstringBounds boundser(font_xform);
+
+    int x_height = font_x_height(finfo, font_xform);
+    double slant = font_slant(finfo);
+    int boundary_char = dvipsenc.boundary_char();
+    
+    int bounds[4], width;
+    
+    for (int code = 0; code < 256; code++) {
+	int g = metrics.glyph(code);
+	if (g != 0 && g != Metrics::VIRTUAL_GLYPH && code != boundary_char) {
+	    if (char_bounds(bounds, width, finfo, font_xform, code)) {
+		int left_sb = (bounds[0] < 0 ? -bounds[0] : 0);
+		metrics.add_single_positioning(code, left_sb, 0, left_sb);
+		
+		if (skew_char >= 0) {
+		    double virtual_height = (bounds[3] > x_height ? bounds[3] :x_height) - 0.5 * x_height;
+		    int right_sb = (bounds[2] > width ? bounds[2] - width : 0);
+		    int skew = (int)(slant * virtual_height + left_sb - right_sb);
+		    metrics.add_kern(code, skew_char, skew);
+		}
+	    }
+	}
+    }
+}
+
 static void
 do_file(const String &otf_filename, const OpenType::Font &otf,
 	const DvipsEncoding &dvipsenc_in, bool dvipsenc_literal,
@@ -1239,6 +1329,14 @@ do_file(const String &otf_filename, const OpenType::Font &otf,
 	return;
     }
 
+    OpenType::Cmap cmap(otf.table("cmap"), errh);
+    assert(cmap.ok());
+
+    FontInfo finfo;
+    finfo.otf = &otf;
+    finfo.cff = font;
+    finfo.cmap = &cmap;
+    
     // save glyph names
     Vector<PermString> glyph_names;
     font->glyph_names(glyph_names);
@@ -1260,17 +1358,14 @@ do_file(const String &otf_filename, const OpenType::Font &otf,
     // initialize encoding
     DvipsEncoding dvipsenc(dvipsenc_in); // make copy
     Metrics metrics(font, font->nglyphs());
-    OpenType::Cmap cmap(otf.table("cmap"), errh);
-    assert(cmap.ok());
+    // encode boundary glyph at 256; pretend its Unicode value is '\n'
+    metrics.encode(256, '\n', metrics.boundary_glyph());
     if (dvipsenc_literal)
 	dvipsenc.make_metrics(metrics, cmap, font, 0, true, errh);
     else {
-	T1Secondary secondary(font, cmap);
-	secondary.set_font_information(font_name, otf, otf_filename);
+	T1Secondary secondary(finfo, font_name, otf_filename);
 	dvipsenc.make_metrics(metrics, cmap, font, &secondary, false, errh);
     }
-    // encode boundary glyph at 256; pretend its Unicode value is '\n'
-    metrics.encode(256, '\n', metrics.boundary_glyph());
 
     // maintain statistics about features
     HashMap<uint32_t, int> feature_usage(0);
@@ -1303,8 +1398,9 @@ do_file(const String &otf_filename, const OpenType::Font &otf,
 	errh->warning("GPOS '%s' error, continuing", e.description.c_str());
     }
 
-    // apply LIGKERN kerning commands to the result
+    // apply LIGKERN kerning and POS positioning commands to the result
     dvipsenc.apply_ligkern_kern(metrics, errh);
+    dvipsenc.apply_position(metrics, errh);
 
     // remove extra characters
     metrics.cut_encoding(257);
@@ -1324,21 +1420,8 @@ do_file(const String &otf_filename, const OpenType::Font &otf,
     }
 
     // apply math letterspacing, if any
-    if (math_spacing) {
-	Transform font_xform;
-	CharstringBounds boundser(font_xform);
-	int bounds[4], width;
-	for (int code = 0; code < 256; code++) {
-	    int g = metrics.glyph(code);
-	    if (g != 0 && g != Metrics::VIRTUAL_GLYPH && code != boundary_char) {
-		if (char_bounds(bounds, width, font, cmap, code, font_xform)) {
-		    int left_sb = (bounds[0] < 0 ? -bounds[0] : 0);
-		    int right_sb = (bounds[2] > width ? bounds[2] - width : 0);
-		    metrics.add_single_positioning(code, left_sb, 0, left_sb + right_sb);
-		}
-	    }
-	}
-    }
+    if (math_spacing)
+	do_math_spacing(metrics, finfo, dvipsenc);
 
     // reencode right components of boundary_glyph as boundary_char
     if (metrics.reencode_right_ligkern(256, boundary_char) > 0
@@ -1388,7 +1471,7 @@ do_file(const String &otf_filename, const OpenType::Font &otf,
     // output
     ::otf_filename = otf_filename;
     output_metrics(metrics, font->font_name(), dvipsenc.boundary_char(),
-		   otf, font,
+		   finfo,
 		   out_encoding_name, out_encoding_file,
 		   font_name, main_dvips_map, errh);
     
@@ -1428,7 +1511,7 @@ main(int argc, char *argv[])
     Clp_AddType(clp, CHAR_OPTTYPE, 0, clp_parse_char, 0);
     program_name = Clp_ProgramName(clp);
 #if HAVE_KPATHSEA
-    kpsei_init(argv[0]);
+    kpsei_init(argv[0], "lcdftools");
 #endif
 #ifdef HAVE_CTIME
     {
@@ -1442,11 +1525,13 @@ main(int argc, char *argv[])
     
     ErrorHandler *errh = ErrorHandler::static_initialize(new FileErrorHandler(stderr, String(program_name) + ": "));
     const char *input_file = 0;
-    const char *glyphlist_file = SHAREDIR "/glyphlist.txt";
+    Vector<String> glyphlist_files;
     bool literal_encoding = false;
     Vector<String> ligkern;
+    Vector<String> pos;
     Vector<String> unicoding;
     bool no_ecommand = false, default_ligkern = true;
+    int warn_missing = -1;
     String codingscheme;
 
     GlyphFilter current_substitution_filter;
@@ -1560,6 +1645,11 @@ main(int argc, char *argv[])
 
 	  case MATH_SPACING_OPT:
 	    math_spacing = !clp->negated;
+	    if (math_spacing && clp->have_arg) {
+		if (clp->val.i < 0 || clp->val.i > 255)
+		    usage_error(errh, "--math-spacing skew character must be between 0 and 255");
+		skew_char = clp->val.i;
+	    }
 	    break;
 	    
 	  case DESIGN_SIZE_OPT:
@@ -1574,6 +1664,14 @@ main(int argc, char *argv[])
 	    ligkern.push_back(clp->arg);
 	    break;
 
+	  case POSITION_OPT:
+	    pos.push_back(clp->arg);
+	    break;
+
+	  case WARN_MISSING_OPT:
+	    warn_missing = !clp->negated;
+	    break;
+
 	  case NO_ECOMMAND_OPT:
 	    no_ecommand = true;
 	    break;
@@ -1708,7 +1806,7 @@ main(int argc, char *argv[])
 	    break;
 
 	  case GLYPHLIST_OPT:
-	    glyphlist_file = clp->arg;
+	    glyphlist_files.push_back(clp->arg);
 	    break;
 	    
 	  case QUERY_FEATURES_OPT:
@@ -1723,7 +1821,9 @@ main(int argc, char *argv[])
 	    if (clp->negated)
 		errh = ErrorHandler::default_handler();
 	    else
-		errh = ErrorHandler::silent_handler();
+		// 9.Nov.05 -- need a new SilentErrorHandler, because we use
+		// the base SilentErrorHandler elsewhere to ignore errors
+		errh = new SilentErrorHandler;
 	    break;
 
 	  case VERBOSE_OPT:
@@ -1811,9 +1911,30 @@ particular purpose.\n");
 	std::sort(interesting_features.begin(), interesting_features.end());
 	std::sort(altselector_features.begin(), altselector_features.end());
 
+	// find glyphlist
+	if (!glyphlist_files.size()) {
+#if HAVE_KPATHSEA
+	    if (String g = kpsei_find_file("glyphlist.txt", KPSEI_FMT_MAP)) {
+		glyphlist_files.push_back(g);
+		if (verbose)
+		    errh->message("glyphlist.txt found with kpathsea at %s", g.c_str());
+	    } else
+#endif
+		glyphlist_files.push_back(SHAREDIR "/glyphlist.txt");
+#if HAVE_KPATHSEA
+	    if (String g = kpsei_find_file("texglyphlist.txt", KPSEI_FMT_MAP)) {
+		glyphlist_files.push_back(g);
+		if (verbose)
+		    errh->message("texglyphlist.txt found with kpathsea at %s", g.c_str());
+	    } else
+#endif
+		glyphlist_files.push_back(SHAREDIR "/texglyphlist.txt");
+	}
+	
 	// read glyphlist
-	if (String s = read_file(glyphlist_file, errh, true))
-	    DvipsEncoding::parse_glyphlist(s);
+	for (String *g = glyphlist_files.begin(); g < glyphlist_files.end(); g++)
+	    if (String s = read_file(*g, errh, true))
+		DvipsEncoding::add_glyphlist(s);
 
 	// read encoding
 	DvipsEncoding dvipsenc;
@@ -1837,17 +1958,22 @@ particular purpose.\n");
 
 	// apply default ligkern commands
 	if (default_ligkern)
-	    dvipsenc.parse_ligkern(default_ligkerns, 0, ErrorHandler::silent_handler());
+	    dvipsenc.parse_ligkern(default_ligkerns, 0, ErrorHandler::ignore_handler());
     
 	// apply command-line ligkern commands and coding scheme
 	cerrh.set_landmark("--ligkern command");
 	for (int i = 0; i < ligkern.size(); i++)
 	    dvipsenc.parse_ligkern(ligkern[i], 1, &cerrh);
+	cerrh.set_landmark("--position command");
+	for (int i = 0; i < pos.size(); i++)
+	    dvipsenc.parse_position(pos[i], 1, &cerrh);
 	cerrh.set_landmark("--unicoding command");
 	for (int i = 0; i < unicoding.size(); i++)
 	    dvipsenc.parse_unicoding(unicoding[i], 1, &cerrh);
 	if (codingscheme)
 	    dvipsenc.set_coding_scheme(codingscheme);
+	if (warn_missing >= 0)
+	    dvipsenc.set_warn_missing(warn_missing);
 
 	do_file(input_file, otf, dvipsenc, literal_encoding, errh);
 	
